/*******************************************************************************
 * Copyright (c) 2003 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.hyades.models.common.facades.behavioral;

/**
 * A decision is a type of action that allows different blocks to be invoked 
 * based upon the result of evaluating a condition.
 * 
 * @author jtoomey
 * @author serge
 * @author marcelop
 * @author psun
 * @since 0.0.1
 * 
 */
public interface IDecision extends IAction {
	/**
	 * Sets the condition that is to determine the execution path.
	 * 
	 * @param condition
	 */	
	public void setCondition(ICondition condition);

	/**
	 * Returns the condition that is to determine the execution path.
	 * 
	 * @return ICondition
	 */	
	public ICondition getCondition();

	/**
	 * Sets the block to be invoked when the condition evaluates to <code>true</code>.
	 * 
	 * @param block
	 */		
	public void setSuccessBlock(IBlock block);

	/**
	 * Returns the block to be invoked when the condition evaluates to <code>true</code>.
	 * 
	 * @return ICondition
	 */	
	public IBlock getSuccessBlock();
	
	/**
	 * Sets the block to be invoked when the condition evaluates to <code>false</code>.
	 * 
	 * @param block
	 */			
	public void setFailureBlock(IBlock block);

	/**
	 * Returns the block to be invoked when the condition evaluates to <code>false</code>.
	 * 
	 * @return ICondition
	 */	
	public IBlock getFailureBlock();
}