/*******************************************************************************
 * Copyright (c) 2003 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.hyades.models.common.facades.behavioral.impl;

import org.eclipse.emf.ecore.resource.Resource;

import org.eclipse.hyades.models.common.facades.behavioral.IBlock;
import org.eclipse.hyades.models.common.facades.behavioral.ICondition;
import org.eclipse.hyades.models.common.facades.behavioral.IDecision;
import org.eclipse.hyades.models.common.facades.behavioral.IExternalImplementor;
import org.eclipse.hyades.models.common.facades.behavioral.ILoop;
import org.eclipse.hyades.models.common.facades.behavioral.ISystemUnderTest;
import org.eclipse.hyades.models.common.facades.behavioral.ITargetInvocation;
import org.eclipse.hyades.models.common.facades.behavioral.ITest;
import org.eclipse.hyades.models.common.facades.behavioral.ITestCase;
import org.eclipse.hyades.models.common.facades.behavioral.ITestComponent;
import org.eclipse.hyades.models.common.facades.behavioral.ITestInvocation;
import org.eclipse.hyades.models.common.facades.behavioral.ITestSuite;
import org.eclipse.hyades.models.common.facades.behavioral.IVerificationPoint;
import org.eclipse.hyades.models.common.fragments.BVRInteraction;
import org.eclipse.hyades.models.common.fragments.Common_Behavior_FragmentsFactory;
import org.eclipse.hyades.models.common.fragments.impl.BVRCombinedFragmentImpl;
import org.eclipse.hyades.models.common.fragments.impl.BVRInteractionConstraintImpl;
import org.eclipse.hyades.models.common.fragments.impl.BVRInteractionOperandImpl;
import org.eclipse.hyades.models.common.fragments.impl.Common_Behavior_FragmentsFactoryImpl;
import org.eclipse.hyades.models.common.interactions.BVRLifeline;
import org.eclipse.hyades.models.common.interactions.Common_Behavior_InteractionsFactory;
import org.eclipse.hyades.models.common.testprofile.Common_TestprofileFactory;
import org.eclipse.hyades.models.common.testprofile.TPFBehavior;
import org.eclipse.hyades.models.common.testprofile.TPFTestCase;
import org.eclipse.hyades.models.common.testprofile.TPFTestComponent;
import org.eclipse.hyades.models.common.testprofile.TPFTestSuite;
import org.eclipse.hyades.models.common.testprofile.impl.Common_TestprofileFactoryImpl;
import org.eclipse.hyades.models.common.testprofile.impl.TPFTestCaseImpl;
import org.eclipse.hyades.models.common.testprofile.impl.TPFTestSuiteImpl;
import org.eclipse.hyades.models.common.testprofile.impl.TPFValidationActionImpl;

/**
 * Factory that creates the Hyades implementation of the simple facade. 
 * 
 * @author marcelop
 * @author psun
 * @since 0.0.1
 */
public class HyadesFactory
{
	public static final HyadesFactory INSTANCE = new HyadesFactory();
	
	public ITestSuite createTestSuite(Resource resource)
	{
		TPFTestSuiteImpl testSuiteImpl = (TPFTestSuiteImpl)Common_TestprofileFactoryImpl.eINSTANCE.createTPFTestSuite();
		resource.getContents().add(testSuiteImpl);
		return testSuiteImpl;
	}

	// TODO: Check this
	public ITestCase createTestCase(Resource resource)
	{
		TPFTestCaseImpl testCaseImpl = (TPFTestCaseImpl)Common_TestprofileFactoryImpl.eINSTANCE.createTPFTestCase();
		resource.getContents().add(testCaseImpl);
		return testCaseImpl;	
	}

	public IExternalImplementor createExternalImplementor(Resource resource, ITest implementedTest)
	{
		TPFTestComponentInstance testComponentInstance = Common_TestprofileFactoryImpl.eINSTANCE.createTPFTestComponentInstance();
		resource.getContents().add(testComponentInstance);
		initializeTestComponentInstance(testComponentInstance, implementedTest);
		
		return (TPFTestComponentInstanceImpl)testComponentInstance;
	}

	public ITestComponent createTestComponent(Resource resource, ITest implementedTest)
	{
		TPFTestComponentInstance testComponentInstance = Common_TestprofileFactoryImpl.eINSTANCE.createTPFTestComponentInstance();
		resource.getContents().add(testComponentInstance);
		TPFBehavior behavior = initializeTestComponentInstance(testComponentInstance, implementedTest);
		
		BVRInteraction interaction = behavior.getInteraction();		
		if(interaction == null)
		{
			interaction = Common_Behavior_FragmentsFactory.eINSTANCE.createBVRInteraction();
			String interactionResource = resource.getURI().path() + "_interaction"; //+ the default interaction resource file extension 
			//resource.getResourceSet().createResource(interactionResource)
		}

		BVRLifeline  lifeline = Common_Behavior_InteractionsFactory.eINSTANCE.createBVRLifeline();
		interaction.getLifelines().add(lifeline);
		testComponentInstance.setLifeline(lifeline);			

		return (TPFTestComponentInstanceImpl)testComponentInstance;
	}
	
	protected TPFBehavior initializeTestComponentInstance(TPFTestComponentInstance testComponentInstance, ITest implementedTest)
	{
		Resource resource = testComponentInstance.eResource();

		TPFTestComponent hyadesTestComponent = Common_TestprofileFactory.eINSTANCE.createTPFTestComponent();
		String hyadesTestComponentResource = resource.getURI().path() + "_testcomponent"; //+ the default Hyades TestComponent resource file extension
		hyadesTestComponent.getTestComponentInstances().add(testComponentInstance);

		TPFBehavior behavior = null;
		if(implementedTest == null)
		{
			behavior = Common_TestprofileFactory.eINSTANCE.createTPFBehavior();
			hyadesTestComponent.getBehaviors().add(behavior);
		}
		else
		{
			if(implementedTest instanceof TPFTestCase)
			{
				TPFTestCase testCase = (TPFTestCase)implementedTest;
				if(testCase.getBehavior() == null)
					testCase.setBehavior(Common_TestprofileFactory.eINSTANCE.createTPFBehavior());
				behavior = testCase.getBehavior();
				
				testCase.getTestSuite().getTestComponentInstances().add(testComponentInstance);
			}
			else if(implementedTest instanceof TPFTestSuite)
			{
				TPFTestSuite testSuite = (TPFTestSuite)implementedTest;
				if(testSuite.getBehavior() == null)
					testSuite.setBehavior(Common_TestprofileFactory.eINSTANCE.createTPFBehavior());
				behavior = testSuite.getBehavior();
				
				testSuite.getTestComponentInstances().add(testComponentInstance);
			}
		}

		return behavior;
	}

	// TODO: Under Construction		
	public IDecision createDecision(IBlock block)
	{	
		BVRCombinedFragmentImpl testDecisionImpl = (BVRCombinedFragmentImpl)Common_Behavior_FragmentsFactoryImpl.eINSTANCE.createBVRCombinedFragment();
	
		block.getActions().add(testDecisionImpl);

		BVRInteractionOperandImpl successOperand = (BVRInteractionOperandImpl) Common_Behavior_FragmentsFactory.eINSTANCE.createBVRInteractionOperand();
		BVRInteractionOperandImpl failureOperand = (BVRInteractionOperandImpl) Common_Behavior_FragmentsFactory.eINSTANCE.createBVRInteractionOperand();
		
		testDecisionImpl.getInteractionOperands().add(0, successOperand);
		testDecisionImpl.getInteractionOperands().add(1, failureOperand);

		return testDecisionImpl;
	}

	// TODO: Under Construction
	public ILoop createLoop(IBlock block)
	{
		BVRCombinedFragmentImpl testLoopImpl = (BVRCombinedFragmentImpl)Common_Behavior_FragmentsFactoryImpl.eINSTANCE.createBVRCombinedFragment();
	
		block.getActions().add(testLoopImpl);
	
		BVRInteractionOperandImpl loopOperand = (BVRInteractionOperandImpl) Common_Behavior_FragmentsFactory.eINSTANCE.createBVRInteractionOperand();
		testLoopImpl.getInteractionOperands().add(loopOperand);
		
		return testLoopImpl;
	}

	public ICondition createCondition()
	{
		BVRInteractionConstraintImpl testConditionImpl = (BVRInteractionConstraintImpl)Common_Behavior_FragmentsFactoryImpl.eINSTANCE.createBVRInteractionConstraint();
		return testConditionImpl;
	}
	
	public ISystemUnderTest createSystemUnderTest(Resource resource)
	{
		TPFSUTInstanceImpl testSUTInstanceImpl = (TPFSUTInstanceImpl)Common_TestprofileFactoryImpl.eINSTANCE.createTPFSUTInstance();
		resource.getContents().add(testSUTInstanceImpl);
		return testSUTInstanceImpl;
	}

	public IVerificationPoint createVerificationPoint()
	{
		TPFValidationActionImpl testVPImpl = (TPFValidationActionImpl)Common_TestprofileFactoryImpl.eINSTANCE.createTPFValidationAction();
		return testVPImpl;
	}

	// TODO: Needs to be implemented
	public ITargetInvocation createTargetInvocation()
	{
		return null;
	}

	// TODO: Needs to be implemented	
	public ITestInvocation createTestInvocation()
	{
		return null;
	}

	
}